/****************************************************************************
 *
 *  ErrorDispatcher.cc                        XMM Science Analysis Subsystem
 *
 *  Implementation of Error interface; handles errors.
 *
 *  (c) 1998  ESA  -  Written by Marco Beijersbergen
 *                    Inspired by Error written by Jon Brumfit
 *
 *  $Id: ErrorDispatcher.cc,v 1.13 2002/06/27 12:31:31 jbakker Exp $
 *
 *****************************************************************************/

#include "ErrorDispatcher.h"

//================================================================
//  class ErrorDispatcher
//================================================================

ErrorDispatcher::ErrorDispatcher() :
  _handler(&_defaultHandler)
{

}

ErrorDispatcher::~ErrorDispatcher()
{
  /* print statistics */
  std::string statistics=_suppressor.statistics();
  if (!statistics.empty())
    _handler->warning("SummaryOfWarnings",statistics);
  _suppressor.reset();

  /* Print any partially assembled messages */

  static bool done = false;

  if(!done){
    done = true;
    std::string remains=_errstr.str();
    if (!remains.empty()) {
      _handler->error("remainingErrorMessage",
		      std::string("remaining error message '")+remains+'\'');
    }
  }
}


/*** Register an error handler ***/

ErrorHandler* ErrorDispatcher::handler(ErrorHandler* newHandler)
{
  ErrorHandler* oldHandler=_handler;
  if (newHandler) _handler=newHandler;
  return oldHandler;
}


/*** Fatal error: this is so bad, nobody can continue ***/

void ErrorDispatcher::fatal(ErrorHandler::Code n)
{  
  // copy the std::string and reset the pointer, otherwise the exit() will
  // call the destructor and see a non-empty errstr.
  std::string msg = _errstr.str();
  _errstr.str("");

  _handler->fatal(n,msg);
}


/*** Error: I can't continue, but my caller could ***/

void ErrorDispatcher::error(ErrorHandler::Code n)
{
  std::string msg = _errstr.str();
  _errstr.str("");

  _handler->error(n,msg);
}


/*** Warning: report this message, but I will continue ***/

void ErrorDispatcher::warning(ErrorHandler::Code n)
{
  // Warning messages may be suppressed :)
  if (_suppressor.suppress(n)) {
    _errstr.str("");
  } else {
    std::string msg = _errstr.str();
    _errstr.str("");
    _handler->warning(n,msg);
  }
}

/*** Message: only report if verbosity is equal or large ***/

void ErrorDispatcher::message(Layer layer, Verbosity level)
{
  std::string msg = _errstr.str();
  _errstr.str("");

  _handler->message(layer,level,msg);
}

/*** Full options ***/
std::string ErrorDispatcher::asCommandLine()
{
  std::ostringstream ss;
  ss <<  _suppressor.options() << " -V " << _handler->verbosity();
  return ss.str();
}
