// -*-C++-*-
//
// File:      SasException.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 19 13:20:18 MET DST 2000

/**
   namespace SasException:

   Contains all exceptions that are thrown by SAS specific code. Implementers
   that wish to use exceptions, should include this file, and derive their
   own Exceptions classes in the same namespace.

   This way, a meta task can at least catch an Exception with a minimal
   message.

   Example: see the namespace itself

   Note:
     'Error' as well as 'Fatal' are not derived from eachother to garantee
     that the order of catching them does not affect their handling.

     For the same reason, catching the base 'Exception' should always
     be preceded by catching derived objects!
*/

#ifndef SasException_h
#define SasException_h

#include <ErrorHandler.h> // Needed for ErrorHandler::Code

namespace SasException {

  /** class Exception:

      Base class for all exceptions thrown by SAS-code developers
  */
  class Exception {
  public:
    Exception(const std::string& msg) : message(msg) {}
    ~Exception(){}
    std::string message;                         // exception message

  protected:
    Exception() {}                          // default constructor

  private:
    /*
    Exception(const Exception&);            // copy constructor (disabled)
    Exception& operator=(const Exception&); // assignment (disabled)
    */
  };


  /** class Error:

      Used by ErrorHandlers, that throw exceptions
  */
  class Error : public Exception {
  public:
    Error(const std::string&             theName,
	  const ErrorHandler::Code& theLabel,
	  const std::string&             theMessage) :
      Exception(theMessage), name(theName), code(theLabel) {}

    std::string             name;                // Name of the application/library
    ErrorHandler::Code code;                // Label used to identify the error
  private:
    Error();                                // default constructor (disabled)
    /*
    Error(const Error&);                    // copy constructor (disabled)
    Error& operator=(const Error&);         // assignment (disabled)
    */
  };


  /** class Fatal:

      Used by ErrorHandlers, that throw exceptions
  */  
  class Fatal : public Exception {
  public:
    Fatal(const std::string&             theName,
	  const ErrorHandler::Code& theLabel,
	  const std::string&             theMessage) :
      Exception(theMessage), name(theName), code(theLabel) {}

    std::string             name;                // Name of the application/library
    ErrorHandler::Code code;                // Label used to identify the error

  private:
    Fatal();                                // default constructor (disabled)
    /*
    Fatal(const Fatal&);                    // copy constructor (disabled)
    Fatal& operator=(const Fatal&);         // assignment (disabled)
    */
  };

}; // namespace SasException

#endif
